@@ -1,14 +1,14 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2010 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2011 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
  * This software is the proprietary information of Tanuki Software.
  * You shall use it only in accordance with the terms of the
  * license agreement you entered into with Tanuki Software.
- * http://wrapper.tanukisoftware.org/doc/english/licenseOverview.html
+ * http://wrapper.tanukisoftware.com/doc/english/licenseOverview.html
  */
 
 /**
@@ -29,13 +29,13 @@ public class WrapperWin32Group
     /*---------------------------------------------------------------
      * Constructors
      *-------------------------------------------------------------*/
-    WrapperWin32Group( byte[] sid, byte[] user, byte[] domain )
+    WrapperWin32Group( String sid, String user, String domain )
     {
         super( user );
         
         // Decode the parameters using the default system encoding.
-        m_sid = new String( sid );
-        m_domain = new String( domain );
+        m_sid = sid;
+        m_domain = domain;
     }
     
     /*---------------------------------------------------------------
