@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.demo;
 
 /*
- * Copyright (c) 1999, 2010 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2011 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -9,48 +9,22 @@ package org.tanukisoftware.wrapper.demo;
  * You shall use it only in accordance with the terms of the
  * license agreement you entered into with Tanuki Software.
  * http://wrapper.tanukisoftware.com/doc/english/licenseOverview.html
- * 
- * 
- * Portions of the Software have been derived from source code
- * developed by Silver Egg Technology under the following license:
- * 
- * Copyright (c) 2001 Silver Egg Technology
- * 
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without 
- * restriction, including without limitation the rights to use, 
- * copy, modify, merge, publish, distribute, sub-license, and/or 
- * sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following 
- * conditions:
- * 
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
  */
 
 import java.awt.Color;
-import java.awt.Font;
-import java.awt.Window;
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.PrintStream;
 
 import javax.swing.JOptionPane;
-import javax.swing.text.AttributeSet;
 import javax.swing.text.SimpleAttributeSet;
 import javax.swing.text.StyleConstants;
-import javax.swing.text.StyleContext;
-import javax.swing.text.AttributeSet.ColorAttribute;
-import javax.swing.text.AttributeSet.FontAttribute;
-
 import org.tanukisoftware.wrapper.WrapperActionServer;
 import org.tanukisoftware.wrapper.WrapperJNIError;
 import org.tanukisoftware.wrapper.WrapperLicenseError;
 import org.tanukisoftware.wrapper.WrapperManager;
 import org.tanukisoftware.wrapper.WrapperListener;
-import org.tanukisoftware.wrapper.WrapperProcess;
 import org.tanukisoftware.wrapper.WrapperProcessConfig;
 import org.tanukisoftware.wrapper.WrapperResources;
 import org.tanukisoftware.wrapper.WrapperSystemPropertyUtil;
@@ -79,6 +53,7 @@ public class DemoApp implements WrapperListener
     private static Process m_testCase;
     private static PrintStream m_childPrintStream;
     private static WrapperResources m_res;
+    private String m_confFile;
 
     /*---------------------------------------------------------------
      * Constructors
@@ -86,6 +61,7 @@ public class DemoApp implements WrapperListener
     private DemoApp()
     {
         m_isTestCaseRunning = false;
+        m_confFile = "../conf/demoapp.conf";
 
     }
 
@@ -160,15 +136,8 @@ public class DemoApp implements WrapperListener
     public Integer start( String[] args )
     {
         String command;
-        System.out.println( "Demo: start()" );
-        /*for ( int i = 0; i < args.length; i++ )
-        {
-            System.out.print( args[i] + " " );
-        }
-*/
         if ( args.length <= 0 )
         {
-            System.out.println( getRes().getString( "Demo: An action was not specified.  Default to \"dialog\"." ) );
             command = "dialog";
         }
         else
@@ -177,6 +146,7 @@ public class DemoApp implements WrapperListener
         }
         if ( command.equals( "dialog" ) )
         {
+            System.out.println( "Demo: start()" );
             System.out.println( getRes().getString( "Demo: Showing dialog..." ) );
             try
             {
@@ -198,7 +168,7 @@ public class DemoApp implements WrapperListener
                 System.out.println( getRes().getString( "Demo: ERROR - Unable to display the GUI:" ) );
                 System.out.println( "Demo:           " + e.toString() );
                 System.out.println( "Demo: " );
-                System.out.println( getRes().getString( "Demo: Fall back to the \"console\" action." ) );
+                System.out.println( getRes().getString( "Demo: This demo requires a display to show its GUI.  Exiting..." ) );
                 command = "console";
             }
             catch ( java.awt.AWTError e )
@@ -207,7 +177,7 @@ public class DemoApp implements WrapperListener
                 System.out.println( getRes().getString( "Demo: ERROR - Unable to display the GUI:" ) );
                 System.out.println( "Demo:           " + e.toString() );
                 System.out.println( "Demo: " );
-                System.out.println( getRes().getString( "Demo: Fall back to the \"console\" action." ) );
+                System.out.println( getRes().getString( "Demo: This demo requires a display to show its GUI.  Exiting..." ) );
                 command = "console";
             }
             catch ( java.lang.UnsupportedOperationException e )
@@ -220,7 +190,7 @@ public class DemoApp implements WrapperListener
                     System.out.println( getRes().getString( "Demo: ERROR - Unable to display the GUI:" ) );
                     System.out.println( "Demo:           " + e.toString() );
                     System.out.println( "Demo: " );
-                    System.out.println( getRes().getString( "Demo: Fall back to the \"console\" action." ) );
+                    System.out.println( getRes().getString( "Demo: This demo requires a display to show its GUI.  Exiting..." ) );
                     command = "console";
                 }
                 else
@@ -229,6 +199,167 @@ public class DemoApp implements WrapperListener
                 }
             }
         }
+        else if ( command.equals( "start" ) )
+        {
+            Thread commandRunner = new Thread( "DemoApp-Command-Runner" ) {
+                public void run() {
+                    BufferedReader br = new BufferedReader( new InputStreamReader( System.in ) );
+                    String command = "";
+                    System.out.println( getRes().getString( "Started and waiting for a command from the Demo Application" ) );
+                    while ( command.compareToIgnoreCase( "finish" ) != 0 )
+                    {
+                        try
+                        {
+                            command = br.readLine();
+                            System.out.println( getRes().getString( "Read action: {0}", command ) );
+        
+                            if ( command.equals( "crash" ) )
+                            {
+                                Thread.sleep( 5000 );
+                                System.out.println( getRes().getString( "Going to cause the JVM to crash." ) );
+        
+                                WrapperManager.accessViolationNative();
+                            }
+                            else if ( command.equals( "frozen" ) )
+                            {
+                                WrapperManager.appearHung();
+                                System.out.println( getRes().getString( "Waiting until wrapper stops this JVM." ) );
+                                while ( true )
+                                {
+                                }
+                            }
+                            else if ( command.equals( "out_of_mem" ) )
+                            {
+                                int ii = 5 - 3;
+                                System.out.println( getRes().getString( "Going to cause a Out of Memory Error" ) );
+                                Thread.sleep( 5000 );
+                                // System.out.println("got java.lang.OutOfMemoryError");
+                                if ( 5 > ii )
+                                {
+                                    getOutOfMemError();
+                                }
+                                System.out.println( getRes().getString( "Application should get restarted now..." ) );
+                                try
+                                {
+                                    Thread.sleep( 5000 );
+                                }
+                                catch ( InterruptedException e )
+                                {
+                                    // TODO Auto-generated catch block
+                                    e.printStackTrace();
+                                }
+                            }
+                            else if ( command.equals( "deadlock" ) )
+                            {
+                                if ( WrapperManager.isStandardEdition() )
+                                {
+                                    System.out.println( getRes().getString( "Deadlock Tester Running..." ) );
+                                    Object obj1 = new Object();
+                                    Object obj2 = new Object();
+                                    int exitCode = 1;
+                                    DeadLock dl = new DeadLock( 1, obj1, obj2 );
+                                    switch ( exitCode )
+                                    {
+                                        case 1:
+                                            System.out.println( getRes().getString( "2-object deadlock." ) );
+                                            dl.create2ObjectDeadlock();
+                                            break;
+                                        case 2:
+                                            System.out.println( getRes().getString( "Wait then 2-object deadlock." ) );
+                                            try
+                                            {
+                                                Thread.sleep( 10000 );
+                                            }
+                                            catch ( InterruptedException e )
+                                            {
+                                            }
+                                            dl.create2ObjectDeadlock();
+                                            break;
+                                        case 3:
+                                            System.out.println( getRes().getString( "3-object deadlock." ) );
+                                            dl.create3ObjectDeadlock();
+                                            break;
+            
+                                        default:
+                                            System.out.println( getRes().getString( "Done." ) );
+                                    }
+                                    // Always wait a couple seconds to make sure the above
+                                    // threads have time to start.
+                                    try
+                                    {
+                                        System.out.println( getRes().getString( "Sleeping for 5 sec..." ) );
+                                        Thread.sleep( 5000 );
+                                    }
+                                    catch ( InterruptedException e )
+                                    {
+                                    }
+                                    System.out.println( getRes().getString( "Main Complete." ) );
+                                }
+                                else
+                                {
+                                    System.out.println( getRes().getString( "Deadlock checks require at least the Standard Edition." ) );
+                                }
+                            }
+                            else if ( command.indexOf( "exec" ) == 0 )
+                            {
+                                try
+                                {
+                                    String input = command.substring(5);
+                                    System.out.println( getRes().getString( "Starting a simple application: " ) + input );
+                                   
+                                    if ( input != null && input.length() > 0 )
+                                    {
+                                        WrapperManager.exec( input, new WrapperProcessConfig().setDetached( false ) );
+                                        System.out.println( getRes().getString( "Successfully executed!") );
+                                    }
+                                }
+                                catch ( SecurityException e )
+                                {
+                                    e.printStackTrace();
+                                }
+                                catch ( NullPointerException e )
+                                {
+                                    e.printStackTrace();
+                                }
+                                catch ( IllegalArgumentException e )
+                                {
+                                    e.printStackTrace();
+                                }
+                                catch ( UnsatisfiedLinkError e )
+                                {
+                                    e.printStackTrace();
+                                }
+                                catch ( IOException e )
+                                {
+                                    e.printStackTrace();
+                                }
+                                catch ( WrapperJNIError e )
+                                {
+                                    e.printStackTrace();
+                                }
+                                catch ( WrapperLicenseError e )
+                                {
+                                    e.printStackTrace();
+                                }
+                            }
+        
+                        }
+                        catch ( IOException e )
+                        {
+                            // TODO Auto-generated catch block
+                            e.printStackTrace();
+                        }
+                        catch ( InterruptedException e )
+                        {
+                            // TODO Auto-generated catch block
+                            e.printStackTrace();
+                        }
+                    }
+                }
+            };
+            commandRunner.start();
+            
+        }
         return null;
     }
 
@@ -262,7 +393,7 @@ public class DemoApp implements WrapperListener
     private Process callAction( String action ) throws IOException
     {
         Process p = null;
-
+        
         if ( action.equals( "mail" ) )
         {
             MailDialog md = new MailDialog();
@@ -285,10 +416,19 @@ public class DemoApp implements WrapperListener
                     // TODO Auto-generated catch block
                     e.printStackTrace();
                 }
-                String arg = "../bin/wrapper -c ../conf/demoApp.conf wrapper.console.flush=TRUE wrapper.console.format=LPM  wrapper.app.parameter.1=start " + md.getEvents()
+                String wrapperBin;
+                if ( WrapperManager.isWindows() )
+                {
+                    wrapperBin = "../bin/wrapper.exe";
+                }
+                else
+                {
+                    wrapperBin = "../bin/wrapper";
+                }
+                String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM  wrapper.app.parameter.1=start " + md.getEvents()
                         + " wrapper.event.default.email.debug=TRUE wrapper.event.default.email.smtp.host=" + md.getServer() + " wrapper.event.default.email.smtp.port="
                         + md.getPort() + " wrapper.event.default.email.sender=" + md.getSender() + " wrapper.event.default.email.recipient=" + md.getRecipients();
-              //  System.out.println( "execing: " + arg );
+                // System.out.println( "execing: " + arg );
                 p = Runtime.getRuntime().exec( arg );
                 m_childPrintStream = new PrintStream( p.getOutputStream() );
 
@@ -355,8 +495,7 @@ public class DemoApp implements WrapperListener
         }
         else if ( action.equals( "service" ) )
         {
-
-            p = Runtime.getRuntime().exec( "..\\bin\\wrapper -it ..\\conf\\demoApp.conf wrapper.console.flush=TRUE" );
+            p = Runtime.getRuntime().exec( "..\\bin\\wrapper.exe -it " + m_confFile + " wrapper.console.flush=TRUE" );
             // WrapperManager.exec( "sudo ../test/demoapp remove" );
             if ( p != null )
             {
@@ -402,20 +541,24 @@ public class DemoApp implements WrapperListener
                 } 
                 p = null;
             }
-
-            // p = WrapperManager.exec( "sudo ../test/demoapp install" );
-
-            // WrapperManager.exec( "sudo ../test/demoapp remove" );
-
         }
         else if ( action.equals( "start" ) )
         {
-            String arg = "../bin/wrapper -c ../conf/demoApp.conf wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=" + action;
-          //  System.out.println( "calling: " + arg );
+            String wrapperBin;
+            if ( WrapperManager.isWindows() )
+            {
+                wrapperBin = "../bin/wrapper.exe";
+            }
+            else
+            {
+                wrapperBin = "../bin/wrapper";
+            }
+            String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=" + action;
+            // System.out.println( "calling: " + arg );
 
             if ( !m_isTestCaseRunning )
             {
-               // System.out.println( "execing: " + arg );
+                // System.out.println( "execing: " + arg );
                 p = Runtime.getRuntime().exec( arg );
 
                 m_childPrintStream = new PrintStream( p.getOutputStream() );
@@ -428,30 +571,94 @@ public class DemoApp implements WrapperListener
                 Runnable setTextRun = new LoggerThread( br, this );
                 Thread t = new Thread( setTextRun );
                 t.start();
-                // return true;
             }
             // ps.close();
         }
         else if ( action.equals( "finish" ) )
         {
+            if ( m_isTestCaseRunning )
+            {
+                m_childPrintStream.println( action );
+    
+                m_childPrintStream.flush();
+                m_childPrintStream.close();
+                // ps.close();
+            }
+        }
+        else if ( action.equals( "exec" ) )
+        { 
+ 
+            String input = "";
+            if ( System.getProperty( "os.name" ).indexOf( "Windows" ) >= 0 )
+            {
+                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the Command, you wish to execute:" ),
+                          getRes().getString( "Child Process Execution" ), JOptionPane.QUESTION_MESSAGE, null, null,
+                          ( Object )"notepad" );
+            }
+            else
+            {
+                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the Command, you wish to execute:" ),
+                          getRes().getString( "Child Process Execution" ), JOptionPane.QUESTION_MESSAGE, null, null,
+                          ( Object )"xclock" );
+            }
+            if ( input != null && input.length() > 0 )
+            {
+                String wrapperBin;
+                if ( WrapperManager.isWindows() )
+                {
+                    wrapperBin = "../bin/wrapper.exe";
+                }
+                else
+                {
+                    wrapperBin = "../bin/wrapper";
+                }
+                String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=start";
+                //System.out.println( "calling: " + arg );
+                if ( !m_isTestCaseRunning )
+                {
+                    p = Runtime.getRuntime().exec( arg );
 
-            m_childPrintStream.println( action );
+                    m_childPrintStream = new PrintStream( p.getOutputStream() );
+                    DemoApp.m_frame.getJMenuBar().getMenu( 0 ).getItem( 0 ).setEnabled( false );
+                    DemoApp.m_frame.getJMenuBar().getMenu( 0 ).getItem( 1 ).setEnabled( true );
+                }
+                if ( p != null )
+                {
+                    m_frame.jTabbedPane2.setSelectedIndex( 1 );
+                    m_isTestCaseRunning = true;
+                    BufferedReader br = new BufferedReader( new InputStreamReader( p.getInputStream() ) );
+                    Runnable setTextRun = new LoggerThread( br, this );
+                    Thread t = new Thread( setTextRun );
+                    t.start();
+                    // return true;
+                }
+
+                m_childPrintStream.println( action + " " + input);
+
+                m_childPrintStream.flush();
+            }
 
-            m_childPrintStream.flush();
-            m_childPrintStream.close();
-            // ps.close();
         }
         else
         {
-            String arg = "../bin/wrapper -c ../conf/demoApp.conf wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=start";
+            String wrapperBin;
+            if ( WrapperManager.isWindows() )
+            {
+                wrapperBin = "../bin/wrapper.exe";
+            }
+            else
+            {
+                wrapperBin = "../bin/wrapper";
+            }
+            String arg = wrapperBin + " -c " + m_confFile + " wrapper.console.flush=TRUE wrapper.console.format=LPM wrapper.app.parameter.1=start";
             //System.out.println( "calling: " + arg );
             if ( !m_isTestCaseRunning )
             {
                 p = Runtime.getRuntime().exec( arg );
 
                 m_childPrintStream = new PrintStream( p.getOutputStream() );
-                this.m_frame.getJMenuBar().getMenu( 0 ).getItem( 0 ).setEnabled( false );
-                this.m_frame.getJMenuBar().getMenu( 0 ).getItem( 1 ).setEnabled( true );
+                DemoApp.m_frame.getJMenuBar().getMenu( 0 ).getItem( 0 ).setEnabled( false );
+                DemoApp.m_frame.getJMenuBar().getMenu( 0 ).getItem( 1 ).setEnabled( true );
             }
             if ( p != null )
             {
@@ -514,165 +721,6 @@ public class DemoApp implements WrapperListener
     {
         System.out.println( getRes().getString( "DemoApp: Initializing..." ) );
         WrapperManager.start( ( new DemoApp() ), args );
-        if ( args.length > 0 && args[args.length - 1].compareTo( "start" ) == 0 )
-        {
-            BufferedReader br = new BufferedReader( new InputStreamReader( System.in ) );
-            String command = "";
-            System.out.println( getRes().getString( "Started and waiting for a command from the Demo Application" ) );
-            while ( command.compareToIgnoreCase( "finish" ) != 0 )
-            {
-                try
-                {
-                    command = br.readLine();
-                    System.out.println( getRes().getString( "Read action: {0}", command ) );
-
-                    if ( command.equals( "crash" ) )
-                    {
-                        Thread.sleep( 5000 );
-                        System.out.println( getRes().getString( "Going to cause the JVM to crash." ) );
-
-                        WrapperManager.accessViolationNative();
-                    }
-                    else if ( command.equals( "frozen" ) )
-                    {
-                        WrapperManager.appearHung();
-                        System.out.println( getRes().getString( "Waiting until wrapper stops this jvm." ) );
-                        while ( true )
-                        {
-                        }
-                    }
-                    else if ( command.equals( "out_of_mem" ) )
-                    {
-                        int ii = 5 - 3;
-                        System.out.println( getRes().getString( "Going to cause a Out of Memory Error" ) );
-                        Thread.sleep( 5000 );
-                        // System.out.println("got java.lang.OutOfMemoryError");
-                        if ( 5 > ii )
-                        {
-                            getOutOfMemError();
-                        }
-                        System.out.println( getRes().getString( "Application should get restarted now..." ) );
-                        try
-                        {
-                            Thread.sleep( 5000 );
-                        }
-                        catch ( InterruptedException e )
-                        {
-                            // TODO Auto-generated catch block
-                            e.printStackTrace();
-                        }
-                    }
-                    else if ( command.equals( "deadlock" ) )
-                    {
-                        System.out.println( getRes().getString( "Dead Lock Tester Running..." ) );
-                        Object obj1 = new Object();
-                        Object obj2 = new Object();
-                        int exitCode = 1;
-                        DeadLock dl = new DeadLock( 1, obj1, obj2 );
-                        switch ( exitCode )
-                        {
-                            case 1:
-                                System.out.println( getRes().getString( "2-object dead lock." ) );
-                                dl.create2ObjectDeadlock();
-                                break;
-                            case 2:
-                                System.out.println( getRes().getString( "Wait then 2-object dead lock." ) );
-                                try
-                                {
-                                    Thread.sleep( 10000 );
-                                }
-                                catch ( InterruptedException e )
-                                {
-                                }
-                                dl.create2ObjectDeadlock();
-                                break;
-                            case 3:
-                                System.out.println( getRes().getString( "3-object dead lock." ) );
-                                dl.create3ObjectDeadlock();
-                                break;
-
-                            default:
-                                System.out.println( getRes().getString( "Done." ) );
-                        }
-                        // Always wait a couple seconds to make sure the above
-                        // threads have time to start.
-                        try
-                        {
-                            System.out.println( getRes().getString( "Sleeping for 5 sec..." ) );
-                            Thread.sleep( 5000 );
-                        }
-                        catch ( InterruptedException e )
-                        {
-                        }
-                        System.out.println( getRes().getString( "Main Complete." ) );
-                    }
-                    else if ( command.equals( "exec" ) )
-                    {
-                        try
-                        {
-                            System.out.println( getRes().getString( "Starting a simple application." ) );
-                            String input = "";
-                           
-                            if ( System.getProperty( "os.name" ).indexOf( "Windows" ) >= 0 )
-                            {
-                                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the Command, you wish to execute:" ), getRes().getString( "Child Process Execution" ), JOptionPane.QUESTION_MESSAGE, null, null,
-                                        ( Object )"notepad" );
-                            }
-                            else
-                            {
-                                
-                                input = ( String )JOptionPane.showInputDialog( m_frame, getRes().getString( "Please enter the Command, you wish to execute:" ), getRes().getString( "Child Process Execution" ), JOptionPane.QUESTION_MESSAGE, null, null,
-                                        ( Object )"xclock" );
-                            }
-                            if ( input != null && input.length() > 0 )
-                            {
-                                WrapperManager.exec( input, new WrapperProcessConfig().setDetached( false ) );
-                                System.out.println( getRes().getString( "Successfully executed!") );
-                            }
-                        }
-                        catch ( SecurityException e )
-                        {
-                            e.printStackTrace();
-                        }
-                        catch ( NullPointerException e )
-                        {
-                            e.printStackTrace();
-                        }
-                        catch ( IllegalArgumentException e )
-                        {
-                            e.printStackTrace();
-                        }
-                        catch ( UnsatisfiedLinkError e )
-                        {
-                            e.printStackTrace();
-                        }
-                        catch ( IOException e )
-                        {
-                            e.printStackTrace();
-                        }
-                        catch ( WrapperJNIError e )
-                        {
-                            e.printStackTrace();
-                        }
-                        catch ( WrapperLicenseError e )
-                        {
-                            e.printStackTrace();
-                        }
-                    }
-
-                }
-                catch ( IOException e )
-                {
-                    // TODO Auto-generated catch block
-                    e.printStackTrace();
-                }
-                catch ( InterruptedException e )
-                {
-                    // TODO Auto-generated catch block
-                    e.printStackTrace();
-                }
-            }
-        }
     }
 }
 
@@ -714,7 +762,7 @@ class LoggerThread implements Runnable
                         String insString = getStyle( str, sas );
                         
                         
-                        m_this.getFrame().getlogTextArea().getDocument().insertString( m_this.getFrame().getlogTextArea().getDocument().getEndPosition().getOffset(), insString + "\n",
+                        m_this.getFrame().getlogTextArea().getDocument().insertString( m_this.getFrame().getlogTextArea().getDocument().getEndPosition().getOffset() - 1, insString + "\n",
                                 sas );
                         int p1 = m_this.getFrame().getlogTextArea().getDocument().getLength();
                         m_this.getFrame().getlogTextArea().setCaretPosition( p1 );
